% include_ass_ind is a binary variable: 0 means we do not want to include
% any of the assembly dummies; 1 means we include some or all of the
% dummies depending on the specification.

% simple_assignment =1: we just assign each model a simple sourcing location: the nearest location to market. Thus, to do this, we revise the data of m structure so that all other locations are NaN
 % =0: we use the original
                                             
function est_results_cost_side = main_costall_log(tar,dum,fxdum,sealandinter, geodummies, include_ass_ind, simple_assignment, demandresult_name)

%% Read in data
% Cost Data
% The data we read in has been generated by DemandToCostAll.do
load costdata
%load demandresult
load(demandresult_name) % name of the demand result, passed as the parameter

%% Define variables
% Define our LHS variable  
m.cost = log(mainout.cost);

% Create dummies: always full set of model and brand dummies, drop one element of other dummies below
m.modelDummies    = zeros(length(m.cost),max(modelCode));
m.marketDummies   = zeros(length(m.cost),max(mktCode));      % That's a country-year code! 
m.brandDummies    = zeros(length(m.cost),max(mainout.m.brandCode));
m.countryDummies  = zeros(length(m.cost),max(mainout.m.ctyCode));
m.yearDummies     = zeros(length(m.cost),5);

for kk = 1 : length(m.cost)
    m.modelDummies(kk,modelCode(kk)) = 1;
    m.marketDummies(kk,mktCode(kk)) = 1;
    m.brandDummies(kk,mainout.m.brandCode(kk)) = 1;
    m.countryDummies(kk,mainout.m.ctyCode(kk)) = 1;
    m.yearDummies(kk,2012-mainout.m.year(kk)) = 1; % Note: order is 2011,2010,2009,2008,..
end

% Generate Brand-country dummies (copied from demand code)
brandInter_temp = [];
for v = 1:size(m.brandDummies,2)
    brandInter_temp = [brandInter_temp bsxfun(@times,m.brandDummies(:,v), m.countryDummies ) ];
end
nonempty_brand = (sum(brandInter_temp,1) > 0)  ;        
m.brandInter = brandInter_temp(:,nonempty_brand); % this is to drop the part of the dummy matrix where a brand is not sold in a country

% drop one element of other dummies 
m.countryDummies(:,9) = []; % Drop USA 
m.yearDummies(:,5)    = [] ; % drop year 2007
m.marketDummies(:,1)  = [];

% Car characteristics
m.sz     = sz;
m.wt     = wt;
m.hp     = hp;
m.mpgcty = mpgcty;
m.hppwt  = hp ./ wt; % could be killed? check

% number of assembly countries
m.numassloc      = numassloc;
m.maxplantmodel  = maxplantmodel; % number of plants for the model with the maximum number of assembly locations, 15

%% Select components of linear cost side (e.g. year dummies, brand dummies, car characteristics)
switch dum
    case 1
        m.linX = [m.modelDummies , m.yearDummies, m.countryDummies, log(m.sz) , log(m.wt), log(m.hp) , log(m.mpgcty) ]; % market dummies, model dummies, model characteristics    
    case 2
        m.linX = [m.modelDummies , m.marketDummies, log(m.sz) , log(m.wt), log(m.hp) , log(m.mpgcty) ]; % market dummies, model dummies, model characteristics    
    case 3
        m.linX = [m.brandDummies , m.marketDummies, log(m.sz) , log(m.wt), log(m.hp) , log(m.mpgcty) ];
    case 4
        m.linX = [m.brandInter, m.yearDummies, log(m.sz) , log(m.wt), log(m.hp) , log(m.mpgcty) ];
end

% These three lines are pre-written here, they will be used in the projection of the linear estimates in the main cost function. No need tochange these when switching between cases
X         = m.linX;
m.XX_inv = eye(size(X,2))/(X'*X);
m.M      =(eye(size(X,1))-X*m.XX_inv*X')'*(eye(size(X,1))-X*m.XX_inv*X'); % if not needed for counterfactuals, kill this later!!!

%% Select nonlinear parameters

% nonlinear stuff (data prepration)
m.distance_assembly_market   = [];
m.distance_assembly_hq       = [];
m.contiguity_assembly_market = [];
m.domestic_assembly_market   = [];
m.tariff_assembly_market     = [];
m.fxrate_assembly            = [];
m.location_code              = []; 
m.land_market_country = [];
for jj = 1:m.maxplantmodel % this is 15!
    m.distance_assembly_market   = [ m.distance_assembly_market   , eval(sprintf('distmarketcountry%d',jj)) ./ 1000];
    m.distance_assembly_hq       = [ m.distance_assembly_hq       , eval(sprintf('disthqcountry%d',jj)) ./ 1000];
    m.contiguity_assembly_market = [ m.contiguity_assembly_market , eval(sprintf('contigmarketcountry%d',jj))];
    m.domestic_assembly_market   = [ m.domestic_assembly_market   , eval(sprintf('domesticmarketcountry%d',jj))];
    m.tariff_assembly_market     = [ m.tariff_assembly_market     , eval(sprintf('tariffmarketcountry%d',jj))];
    m.fxrate_assembly            = [ m.fxrate_assembly            , eval(sprintf('fxrate%d',jj))];
    m.location_code              = [ m.location_code ,  eval(sprintf('iso%dnumeric',jj))];
    m.land_market_country        = [ m.land_market_country ,  eval(sprintf('landmarketcountry%d',jj))];
end    

% This line will simplify extraction of f.e. 
m.location_code(isnan(m.location_code)) = nanmax(m.location_code(:))+1 ;

% set the value for sigma_v
% We have experimented wih alternative values of sigma_v. Sigma_v =0.001
% gives essentially the same estimates. 
m.sigma_v=0.01;

%% Choose the set of location dummies 
% Decide on set of countries to include on the cost side. 
% 1 = production country included
% 0 = the base country (location costs normalized to 0)
% NaN = assembly country dropped

m.include_countries = ones(1,numassloc);
m.include_countries(11) = 0; % normalize Germany

if include_ass_ind == 0 % if we do not want to include any of the assembly location dummies, we set m.include_countries to be NaN for all of them.
    % Note that we are using a full set of model or brand dummies, so we
    % need to drop all the assembly dummies without adding any dummy back.
    m.include_countries = m.include_countries*0; % all zeros: it means that all assembly locations have the same productivity.
end

if simple_assignment == 1 % =1: we just assign each model a simple sourcing location: the nearest location to market. Thus, to do this, we revise the data of m structure so that all other locations are NaN
                                             % =0: we use the original
                                             % model in our draft.
    assembly_mkt_dist_min = min(m.distance_assembly_market,[],2); %nearest distance from mkt to assembly
    nearest_dum = (repmat(assembly_mkt_dist_min,1,size(m.distance_assembly_market,2)) == m.distance_assembly_market); % indicator of nearest assembly      
    % Make changes:
    m.tariff_assembly_market(nearest_dum == 0) = NaN;
    m.distance_assembly_market(nearest_dum == 0) = NaN;
    m.contiguity_assembly_market(nearest_dum == 0) = NaN;
    m.domestic_assembly_market(nearest_dum == 0) = NaN;
    m.distance_assembly_hq(nearest_dum == 0) = NaN;
    m.fxrate_assembly(nearest_dum == 0) = NaN;
    m.land_market_country(nearest_dum == 0) = NaN;
end

% We experimented with dropping these model dummies which were perfectly
% collinear with country dummies and other variables
% m.modelDummies(:,469) =[];
% m.modelDummies(:,444) =[];
% m.modelDummies(:,568) =[];
% m.modelDummies(:,517) =[];
% m.modelDummies(:,471) =[];

% Alternatively, to deal with the same collinearity problem, we may
% normalize the production cost in these countries
%  m.include_countries(9) = 0;
%  m.include_countries(2) = 0; 
%  m.include_countries(34) = 0;
%  m.include_countries(31) = 0;
%  m.include_countries(38) = 0;
%  m.include_countries(21) = 0;

% We experimented with dropping these countries, which basically does not change the estimates at all, as these countries are estimated as very costly
% and with limited data there is an identification problem whether these costs are 5000 or 5 millions, for example. 
% m.include_countries(12)  = NaN;          
% m.include_countries(15)  = NaN;           
% m.include_countries(19) = NaN;          
% m.include_countries(24) = NaN;          
% m.include_countries(32) = NaN;    
% m.include_countries(45) = NaN;             
% m.include_countries(49) = NaN;           
% m.include_countries(47) = NaN;    
% m.include_countries(46) = NaN;       
% m.include_countries(43) = NaN;    
% m.include_countries(25) = NaN;     
% m.include_countries(27) = NaN;  
% m.include_countries(30) = NaN;    
% m.include_countries(42) = NaN;         
% m.include_countries(48) = NaN;            


%% Optimization

% We do a multi-start procedure of the estimation and select the best point
% below
ktropts = optimset('Display','off','GradObj','on','TolFun',1E-6,'TolX',1E-15,'maxit',2000); % Display: final
RUNS = 10;      % was 15       
rng(1024) % set the seed for random multi start
%numcostvar = 4 + tar + fxdum ; % number of geographical variables: 2 distances always in and tariff coefficient depending on tar
numcostvar           = (1) + (tar + fxdum) + ( 1 + sealandinter) + 2*(geodummies==1) ...
                                                                 + 1*(geodummies==2) ...
                                                                 + 2*(geodummies==3) ... % was 3 -- a bug???
                                                                 + 1*(geodummies==5) ...
                                                                 + 3*(geodummies==6); % number of geographical variables

beta_guess_all = [-1 + 2 .* rand(numcostvar,RUNS); zeros(nansum(m.include_countries),RUNS)]; 

parfor run_id=1:RUNS

% if run_id == 1
%     switch tar
%         case 0
%             beta_guess       = [0.1 ; - 0.3 ; - 0.1 ; 0.1 ; 1 ;zeros(nansum(m.include_countries),1)] ;   % Parameters for trade, gamma fc costs function 
%             if fxdum == 0
%                 beta_guess(5)      = [];
%             end
%         case 1  % With coefficient on tariff
%             beta_guess       = [0.1 ; - 0.3 ; - 0.1 ; 0.1 ;1;  1 ; zeros(nansum(m.include_countries),1)] ;   % Parameters for trade, gamma, fc costs function 
%             if fxdum == 0
%                 beta_guess(6)      = [];
%             end
%     end
% else
%     beta_guess       =  [-1 + 2 .* rand(numcostvar,1); zeros(nansum(m.include_countries),1)];   % Parameters for trade costs function    % ~ U(0,2) and U(.5,1.1)
% end

% simply use a ramdon start:
beta_guess       =  beta_guess_all(:,run_id); 

starts(:, run_id)     = beta_guess;

% Gradient check of objective function
% x     = beta_guess;
% h     = eps.^(1/2).*max(abs(x),1);
% xh    = x+h; 
% h     = xh-x;
% [f0, Grad_f_anal]= cost_objective_func_log_gradient(beta_guess,m,tar,fxdum);      
% 
% Grad_f_fd = zeros(size(x));
% for  j = 1:size(x,1);
%    j
%  varargin    = x; 
%  varargin(j) = xh(j); 
%  [f1]= cost_objective_func_log_gradient(varargin,m,tar,fxdum);       
%  Grad_f_fd(j) = (f1(:)-f0(:))/h(j);
% end
% disp('Max absolute difference in Gradients')
% disp(max(abs(Grad_f_anal-Grad_f_fd)))

LB = -inf*ones(size(beta_guess));  % Tariff coefficient log(1+tariffcoef*tariff) is constrained such that there is no negative in the log

if tar==1
    LB(2) = 0;
end

if strcmp(getenv('HOME'),'/home/kcosar')  ==1    % booth servers use old version of knitro
    [Estim,fval,exitflag,output] = ktrlink(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],ktropts);
elseif strcmp(getenv('HOME'),'/home/cosar')  ==1
    [Estim,fval,exitflag,output] = knitromatlab(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],[],ktropts);
elseif strcmp(getenv('HOME'),'/home/tintelnot')  ==1
    [Estim,fval,exitflag,output] = knitromatlab(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],[],ktropts);   
elseif strcmp(getenv('USERNAME'),'kcosar')  ==1   % desktop 
    [Estim,fval,exitflag,output] = knitromatlab(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],[],ktropts);
elseif strcmp(getenv('USERNAME'),'sli')  ==1   % desktop 
    [Estim,fval,exitflag,output] = knitromatlab(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],[],ktropts);
else % so the default is to use knitromatlab
    [Estim,fval,exitflag,output] = knitromatlab(@(x) cost_objective_func_log_gradient(x,m,tar,fxdum,sealandinter, geodummies),beta_guess,[],[],[],[],LB,[],[],[],ktropts);
end

disp(sprintf('Completed iteration %d\n', run_id));

sols(:,run_id)          = Estim;
objective_value(run_id) = fval;
flag(run_id)            = exitflag;
out_status{run_id}      = output;
%save('mstart_output_cos_all_log', 'sols', 'starts','objective_value','flag', 'out_status');   % why save this?

end

% Take the best point
[~, idx]              = min(objective_value);
paraNonlinCost         = sols(:,idx);        % Note: at this point, the nonlinear parameters are still divided by sigma_v. Will adust this below.
%Here run_id check that this point is actually a locally optimal solution...
if (flag(idx) ~= 0) 
    disp(sprintf('WARNING! Exit Flag indicates this point is not locally optimal.  Flag = %d\n',flag(idx)));
end

[fval,grad,paraLinCost,cost_residual,dResidual_dNonlinparams,share_assembly_loc] = cost_objective_func_log_gradient(paraNonlinCost,m,tar,fxdum,sealandinter, geodummies);
% if no locations are dropped, the standard error code will produce a
% warning. See more detailed notes inside getSeCost function.
[se,  var_mat] = getSeCost(paraNonlinCost, m,tar,fxdum,sealandinter, geodummies);       % linear parameters come first, nonlinear parameters at the end.
%disp('....')
%% 
%disp('Cost side estimates of geographic frictions')
%[paraNonlinCost(1:length(beta_guess)) * m.sigma_v,se(size(m.linX,2)+1:size(m.linX,2)+length(beta_guess))  * m.sigma_v]  % Se. contain s.e. of linear parameters. We skip them here.

%% Predict outcomes: note that cost_vector is in log -- there was a bug using cost_vector directly in computing revenue_flows_assembly_market_baseline
[cost_vector, share_assembly] = predict_counterfactual_cost_outcomes_func(cost_residual,paraLinCost,paraNonlinCost,m,tar,fxdum,sealandinter, geodummies);

total_sales                                      = mainout.m.sales;  % comes from the data
revenue_flows_assembly_market_baseline           = bsxfun(@times,total_sales .* exp(cost_vector),share_assembly);           % This is num_obs x numassloc
quantity_flows_assembly_market_baseline          = bsxfun(@times,total_sales ,share_assembly); 

%% Here we do an exercise of estimating parameters, if only the cheapest cost location according to our estimated model was kept.
% this is then a linear problem. We comment this code out below.

% % Predicting trade flows
% [max_share , cheapest_loc] = max(share_assembly_loc,[],2);
%
% pseudolinX_noloc = zeros(size(m.linX,1),4);
% loc_temp = zeros(size(m.linX,1),1);
% tariff_temp = zeros(size(m.linX,1),1);
% 
% for kk = 1:size(m.linX,1)
% pseudolinX_noloc(kk,:) = [log(10*m.distance_assembly_market(kk,cheapest_loc(kk))),m.domestic_assembly_market(kk,cheapest_loc(kk)),m.contiguity_assembly_market(kk,cheapest_loc(kk)),log(10*m.distance_assembly_hq(kk,cheapest_loc(kk)))];
% loc_temp(kk) = m.location_code(kk,cheapest_loc(kk));
% tariff_temp(kk) = log(1+m.tariff_assembly_market(kk,cheapest_loc(kk)));
% end
% 
% pseudelinX_loc = dummyvar(loc_temp);
% KKK = size(pseudelinX_loc,2) ;
% for kk = 1 : KKK 
%     if m.include_countries(KKK+1-kk) ~= 1
%         pseudelinX_loc(:,KKK+1-kk) = [];
%     end
% end
%     
% Stata_Y = m.cost - tariff_temp;
% X_test = [pseudolinX_noloc,pseudelinX_loc,m.linX];
% [beta_linear_simple,stdx] = lscov(X_test,Stata_Y);
% 
% disp('Cost side predictions of geographic frictions by the pseudo linear model')
% 
% [beta_linear_simple(1:4),stdx(1:4)]
% beta_linear_simple_two = (eye(size(X_test,2)) / (X_test' * X_test)) *  (X_test' * Stata_Y)  ;
% %to_stata = [ loc_temp ,  mainout.m.modelCode ,  Stata_Y , X_test ];

%% results
m.M                = [];  % this is the big matrix used in estimation that we don't need any further

m.distinternal     = distinternal;  % will be used in counterfactuals; for each observation, the internal distance of the market
m.distinternalhq   = distinternalhq;

est_results_cost_side.paraNonlinCost     = paraNonlinCost;
est_results_cost_side.paraLinCost        = paraLinCost;
est_results_cost_side.se                 = se;
est_results_cost_side.share_assembly_loc = share_assembly_loc;
est_results_cost_side.fval               = fval;
est_results_cost_side.grad               = grad;
est_results_cost_side.cost_residual      = cost_residual;
est_results_cost_side.var_mat            = var_mat;
est_results_cost_side.flag               = flag(idx);
est_results_cost_side.m                  = m;
est_results_cost_side.tariff             = tar;
est_results_cost_side.dummyCategory      = dum;
est_results_cost_side.fxdum              = fxdum;
est_results_cost_side.sealand      = sealandinter;
est_results_cost_side.geo              = geodummies;
est_results_cost_side.dResidual_dNonlinparams                 = dResidual_dNonlinparams;
est_results_cost_side.total_sales                             = total_sales;
est_results_cost_side.revenue_flows_assembly_market_baseline  = revenue_flows_assembly_market_baseline;
est_results_cost_side.quantity_flows_assembly_market_baseline = quantity_flows_assembly_market_baseline;

% The following function is a replacement of the below commented part, we
% do this since we need to use the same part of the code to report
% bootstrapping result.
[est_results_cost_side.coefs, est_results_cost_side.coefSE] = reportCoef(est_results_cost_side.paraNonlinCost, est_results_cost_side.paraLinCost, est_results_cost_side.se, tar,fxdum, sealandinter, geodummies, est_results_cost_side.m.sigma_v);

% % coefficients and standard errors to be written to output and reported: extract the estimates and standard errors of geographical and characteristics costs
% est_results_cost_side.coefs  =  cat(1,est_results_cost_side.paraNonlinCost(1:numcostvar)*est_results_cost_side.m.sigma_v,...
%     est_results_cost_side.paraLinCost(length(est_results_cost_side.paraLinCost)-3:length(est_results_cost_side.paraLinCost)));
%                    
% est_results_cost_side.coefSE = cat(1,est_results_cost_side.se(length(est_results_cost_side.paraLinCost)+1:length(est_results_cost_side.paraLinCost)+numcostvar)*est_results_cost_side.m.sigma_v,...
%     est_results_cost_side.se(length(est_results_cost_side.paraLinCost)-3:length(est_results_cost_side.paraLinCost)));
%                      
% if tar==0  
%     if fxdum == 0
%         est_results_cost_side.coefs = [est_results_cost_side.coefs(1:4);NaN;NaN;est_results_cost_side.coefs(5:8)];
%         est_results_cost_side.coefSE = [est_results_cost_side.coefSE(1:4);NaN;NaN;est_results_cost_side.coefSE(5:8)];
%     else
%         est_results_cost_side.coefs = [est_results_cost_side.coefs(1:4);NaN;est_results_cost_side.coefs(5:9)];
%         est_results_cost_side.coefSE = [est_results_cost_side.coefSE(1:4);NaN;est_results_cost_side.coefSE(5:9)];
%     end
% elseif tar==1
%     if fxdum == 0
%         est_results_cost_side.coefs  = [est_results_cost_side.coefs(1:4);...
%             est_results_cost_side.coefs(5)/est_results_cost_side.m.sigma_v;...  % since tariff coef is not linear but rather exponential, it doesn't scale here. in the cost_objective_func_log_gradient.m, it is already divided by sigma_v
%             NaN;...
%             est_results_cost_side.coefs(6:9)];
%         est_results_cost_side.coefSE = [est_results_cost_side.coefSE(1:4);...
%             est_results_cost_side.coefSE(5)/est_results_cost_side.m.sigma_v;...
%             NaN;...
%             est_results_cost_side.coefSE(6:9)];
%     else
%         est_results_cost_side.coefs  = [est_results_cost_side.coefs(1:4);...
%             est_results_cost_side.coefs(5)/est_results_cost_side.m.sigma_v;...  % since tariff coef is not linear but rather exponential, it doesn't scale here. in the cost_objective_func_log_gradient.m, it is already divided by sigma_v
%             est_results_cost_side.coefs(6:10)];
%         est_results_cost_side.coefSE = [est_results_cost_side.coefSE(1:4);...
%             est_results_cost_side.coefSE(5)/est_results_cost_side.m.sigma_v;...
%             est_results_cost_side.coefSE(6:10)];
%     end
% end

end