% The purpose of this script is to generate a complete start vector for the
% mpec method which satisfies all constraints from an arbitrary set of
% nonlinear parameters.  It is adapted from constraintTest, but has a
% different purpose. constraintTest includes a Jacobian test and is meant
% for testing, this one hard codes the initial point so that it can be used
% to start multiple starts, also, it will save to "generated" startpoint to
% a specific filename 

% this is revised to be a function. The input is m (or the name of the file
% generated by setupStruct), while the output is a nice guess of theta, to
% be used in MPEC_main. We do not save any file here, but put the guess in
% structure m. (May cause overheat?)
function [theta_start] = genMPECStart(m, parnonlin)

%% STEP 1: Load data and set nonlinear parameters
%clear
% load eurData_sparse
% eval(['load ', toLoadFile, '.mat']); % load file that constructed by setupStruct

%Need to use a default nonlinear parameter
if (nargin < 2)
    %This start point is "inspiried" by a run that started with the multinomial
    %logit, this run moved the Pi parameters but not the sigmas, here we're
    %starting at a sigma of all ones, but we want to solve for delta rather
    %than try and get the MPEC estimator to both find a feasible point and
    %optimize.
    %parnonlin = [ 1 1 1 1 1 0 0 0 0]';
    %parnonlin = [   -2.4215   -1.6221   -0.9837   -2.8012  -19.9805   11.9065  -49.9000    2.6496  -11.4064]';
    %parnonlin = [ 1 1 1 1 1]';
    %parnonlin = [ 5 5 5 5 1 0]';
    %load theta_hat_sigOnly
    %parnonlin = theta_hat(1:5);
    startSig = 3*ones(size(m.SigmaEntries, 1),1);
    if isempty(m.PiEntries)
        startPi = [];
    else
        startPi = zeros(size(m.PiEntries,1),1);
    end
    startAlphaBar = 10; 
    startAlphaPi  = -.8*ones(size(m.AlphaPiEntries, 1),1);
    startAlphaSigma = .5;

    parnonlin = [ startSig; startPi; startAlphaBar; startAlphaPi; startAlphaSigma];
end

mu = getMu(parnonlin,m);

%% STEP 2: Solve constraints to get delta:

%Set up the jacobian pattern....this bit of code should really be
%functionalized someday...
Jac_Pattern_delta = zeros(m.nObs, m.nObs);
for mk=1:m.nMkts
          ind_mk = (m.mktCode==mk);
          Jac_Pattern_delta(ind_mk,ind_mk) = 1;
end

% For Hessian Pattern 
Hessian_Pattern_delta = sparse(m.nObs, m.nObs);
for mk=1:m.nMkts;
          ind_mk = (m.mktCode==mk);
          Hessian_Pattern_delta(ind_mk,ind_mk) = 1;
end           
Hessian_Pattern_delta_sparse = sparse(Hessian_Pattern_delta);

%Clear out the non-essential stuff for maximum available memory
clear Hessian_Pattern

%ktropts = optimset('Display','iter','JacobPattern',Jac_Pattern);
% note that x here is delta
%ktropts = optimset('Display','iter','JacobPattern',Jac_Pattern_delta, 'HessPattern',Hessian_Pattern_delta_sparse, 'Hessian','user-supplied', 'HessFcn', @(x, lambda) getHessian_dumObj(x, lambda, mu, m)  );
ktropts = optimset('Display','iter','JacobPattern',Jac_Pattern_delta  );

guess = m.delta_start;
guess = getDelta(guess, mu, m, 10000, 1e-5);

%[share share_ij] = getShare(guess, mu, m); % add this line, since the input of shareConstraintsParallel.m has changed.
 delta = guess; % use the guess to reduce the usage of KNITRO on cluster

%[delta, fval ,exitflag_delta,output_delta] = knitrolink(@(x) dummy_objective(x,mu,m),guess,[],[],[],[],[],[],@(x) shareConstraintsParallel_dumObj(x, mu,m),ktropts,'knitro_7_shares_full.opt');

%   [delta, fval ,exitflag_delta,output_delta] = knitrolink(@(x) dummy_objective(x),guess,[],[],[],[],[],[], ...
%   @(x) shareConstraintsParallel_dumObj(x, mu,m),ktropts,'knitro_9_dumObj_noHess.opt');

%% STEP 3: Solve the linear constraints. 

[~, G] = getDesign(m);
startg = G*delta;

%% Step 4: Put it all together...and save
theta_start = [ parnonlin; startg; delta];

