%This function generates an (m, c2str) pair using the options/identifiers chosen in setupSpecIdentifiers.m
function [m c2str] =  setupStructs( )

%Set the specification flags
setupSpecIdentifiers;

m.fileVersion = fileVersion; % keep the file version in m

% Model and firm names for reporting
m.model_names_for_markups = model_names_for_markups;
m.firm_names_for_markups  = firm_names_for_markups;
m.model_names_for_elast   = model_names_for_elast;

% specifications
if strcmp(CharSquare,'Squares')
    m.chsq = 1;
elseif strcmp(CharSquare, 'SizeOnly')
    m.chsq = 2;
else
    m.chsq = 0;
end

if strcmp(priceRC,'log_normal')
    m.lognormprice = 1;
else
    m.lognormprice = 0;
end

if strcmp(priceFormat,'Log')
    m.priceinlog = 1;
else
    m.priceinlog = 0;
end

if (strcmp(same_rc_distr,'Same'))
    m.same_rc_distr = 1;
else
    m.same_rc_distr = 0;
end

m.xSet       = xSet;
m.Est_Method = Est_Method;
    

if m.lognormprice==1
%   This is the case where price enters lognormally, needs special treatment
   
   m.SigmaEntries = [ 1 1; 2 2 ; 3 3; 4 4]; % maximum is 4
   %m.SigmaEntries = [ 1 1 ; 3 3; 4 4]; %No hppwt random coefficient, just to see. 
   m.PiEntries = [ 4 1]; %Taste for size varies with income
   m.AlphaPiEntries = [ 1  ];
   % In this code, we fix AlphaBar to be the same for all countries, this may change in the future.
   % THE 1'S represent the length of scalar parameters: alphaBar and alphaSigma.
   m.size_nl = size(m.SigmaEntries,1) + size(m.PiEntries,1) + 1 + length(m.AlphaPiEntries) + 1;
else
   %Here price is just the last entry inthe RC matrix...
   m.SigmaEntries =  [ 1 1 ;  2 2; 3 3; 4 4; 5 5]; % maximum is 5
   m.PiEntries = [ 5 1 ]; % i.e., [4 1 ]; 
   m.AlphaPiEntries = []; % should always be empty since this is standard normal rc, alphaPi is a part of Pi
   m.size_nl = size(m.SigmaEntries,1) + size(m.PiEntries,1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load Data 
% csvread interprets things strangely so use Matlab autogenerated script...
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% this version uses the data set in April which include more countries.
switch fileVersion 
    case 'Full'
         [data, varnames] = readInCsv(['..' filesep 'input_files' filesep 'demandData.csv']);  % this csv file added to more iv's
    case 'noBRA'
         [data, varnames] = readInCsv(['..' filesep 'input_files' filesep 'noBRA_demandData.csv']);  % this csv file added to more iv's
    case 'EU'
         [data, varnames] = readInCsv(['..' filesep 'input_files' filesep 'onlyEU_demandData.csv']);  % this csv file added to more iv's
    otherwise
       fprintf('Error: Unrecognized file version name: %s \n', fileVersion);
   return;
end

for v = varnames
    eval(sprintf('%s = data.%s;', v{1}, v{1}));
end

%clearvars data varnames;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load the cost data set to get cost side IV's and confirm the order is the same as the demand data set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch fileVersion 
    case 'Full'
         [cData, cVars] = readInCsv(['..' filesep 'input_files' filesep 'costData.csv']);  % this csv file added to more iv's
    case 'noBRA'
         [cData, cVars] = readInCsv(['..' filesep 'input_files' filesep 'noBRA_costData.csv']);  % this csv file added to more iv's
    case 'EU'
         [cData, cVars] = readInCsv(['..' filesep 'input_files' filesep 'onlyEU_costData.csv']);  % this csv file added to more iv's
    otherwise
       fprintf('Error: Unrecognized file version name: %s \n', fileVersion);
   return;
end

assert(sum(cData.price ~= price) == 0, 'setupStructs: Cost and Demand Data Files Do Not Match.') % to check that we didn't mismatch set of IVs
iv_mindist_ns = cData.ivmindistns;
iv_log_mindist = cData.ivlogmindist;
iv_mindist_contig = cData.ivmindistcontig;
iv_mindist_domestic = cData.ivmindistdomestic;
iv_mindist_wage = cData.ivmindistwage;
iv_log_mindist_wage = cData.ivlogmindistwage;
iv_log_mindist_tariff = cData.ivlogmindisttariff;
iv_mindist_tariff = cData.ivmindisttariff;

%clearvars cData cVars

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Now, setup code and orginization variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m.nObs = length(modelCode);
m.mktCode = mktCode;
m.ctyCode = ctyCode;
m.firmCode = firmCode;
m.brandCode = brandCode;
m.modelCode = modelCode;
m.brandcty = brandcty;
m.year = data.year;  %Note, use data.year because "year" is a reserved work...oops.
m.sales = sales;

%Figure out number of various things:
m.nFirms = length(unique(m.firmCode));
m.nBrands = length(unique(m.brandCode));
m.nCtys  = length(unique(m.ctyCode));
m.nMkts  = length(unique(m.mktCode));
m.nYears = length(unique(m.year));
m.nModels = length(unique(m.modelCode));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Write c2str so that we can read all the codes.
%   This structure is primarily for debugging, shouldn't be any need
%   to permenantly send it to funcitons. 

%
% This code is very hard to read because it uses eval to loop over the four
% types, the compactness makes it worth it over copy/paste, and I can't
% figure out a better way.
lc = { 'mkt',  'cty',  'brand', 'firm',  'model'};
uc = {'Mkts', 'Ctys', 'Brands', 'Firms', 'Models'};

for v = 1:length(lc)
   eval(['c2str.',uc{v},' = cell(m.n', uc{v}, ',1);']);
   eval(['numTypes = m.n',uc{v},';']);
   for f = 1:numTypes
    eval(['tempcell = unique(',lc{v},'(',lc{v},'Code==f));']);
    if length(tempcell)>1
        fprintf('Warning: %sCode %d is not unique!\n', lc{v} , f);
    end
    
    if length(tempcell) == 1
        eval(['c2str.',uc{v},'(f) = tempcell;']);
    elseif length(tempcell)<1
        fprintf('Warning: %sCode %d does not exist!\n', lc{v} , f);
        eval(['c2str.',uc{v},'(f) = {''MISSING!''};']);
    end
   
    
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Add the actual data...
%%%The SHARES to match....
m.share    = share;
% The outside share, needed only for multionomial logit...
m.outshare = outshare;
%%% NORMALIZATION: Horsepower, size and weight are badly scaled, normalize them by their averages
switch mpSet
    case 'mpg'
          m.unit_adjusted_norm = mean([hppwt sz mpgcty]) .* [100, 1e-6, 1]; % This will be used in printTables_demand only. Note that using this means we use horse power per 100kg
          hpszwtmpg = bsxfun(@rdivide, [hppwt sz mpgcty], mean([hppwt sz mpgcty]));
          blpIV  = [ ivowncon ivownhppwt ivownsize ivownmpgcty ...
           ivrivcon ivrivhppwt ivrivsize ivrivmpgcty];
          
          chars_out_temp = ['..'  filesep 'demand_output' filesep 'chars'];
          save(chars_out_temp,'hpszwtmpg','m','hppwt', 'sz', 'mpgcty', 'c2str')

    case 'mpD' 
         m.unit_adjusted_norm = mean([hppwt sz mpDcty]) .* [100, 1e-6, 1]; % This will be used in printTables_demand only. Note that using this means we use horse power per 100kg
         hpszwtmpg = bsxfun(@rdivide, [hppwt sz mpDcty], mean([hppwt sz mpDcty]));
         hpszwtmpgDEU = bsxfun(@rdivide, [hppwt sz mpDctyDEU], mean([hppwt sz mpDcty])); % for counterfactuals where German gas prices will prevail everywhere
         blpIV  = [ ivowncon ivownhppwt ivownsize ivownmpDcty ...
           ivrivcon ivrivhppwt ivrivsize ivrivmpDcty];
       
        chars_out_temp = ['..'  filesep 'demand_output' filesep 'chars'];
        save(chars_out_temp,'hpszwtmpg','m','hppwt', 'sz', 'mpDcty', 'c2str')
    otherwise
       fprintf('Error: Unrecognized fuel efficiency measure Set Requested: %s \n', xSet);
       return;
end

% save country level sales weighted chars (normalized by mean)
for i = 1:m.nCtys
    char_cty = hpszwtmpg(m.ctyCode == i,:); % the chars of this country, n by 3
    sales_cty = sales(m.ctyCode == i); % sales of this country, n by 1
    char_temp = sum(char_cty .* repmat(sales_cty,1,size(char_cty,2))/sum(sales_cty),1);
    m.char_mu_weighted(:,i) = char_temp';
end

%%% RANDOM COEFFICIENTS:
%Now it's time to define m and own:
%These are the random coefficient parameters (price dealt with separately)
%Note we are adding a constant (stata did this for us.)
%m.rcX = [ones(m.nObs,1) hpszwtmpg];

% Include hppwt, size, and mpD as coefficients. 
m.rcX = [ ones(m.nObs,1), hpszwtmpg(:,[1,2,3])];
m.rcXcounter = [ ones(m.nObs,1), hpszwtmpgDEU(:,[1,2,3])];  % for counterfactuals where German gas prices will prevail everywhere, doesn't seem like this should be done here...


%Assign price and share (easy), we keep price seperate from other random
%coefficients in case we got to log(y-p) specificaiton...
if m.priceinlog == 1
    m.p = log(price);
else
    m.p = price/10000;
end

%If m.lognormprice == 0, price is treated like other RCs, otherwise it is
%handled separately and assumed to be log-normal. 
if (m.lognormprice==0)
    m.rcX = [m.rcX -m.p];
    m.rcP = zeros(size(m.rcX));
    m.rcP(:,end) = ones(size(m.rcX,1),1);
else
    %Note: This is NOT because price doesn't enter as a random coefficient
    %but because it is handled seperately from rcX because it is lognormal,
    %we read it from m.p directly (or user supplies alternative price in
    %counterfactuals).
    m.rcP = zeros(size(m.rcX));
    
end

%% LINEAR COEFFICIENTS
%linX are the linear parameters, they are only used in getDesign.  
%We are changing getDesign, which formerly took a compressed version
%of the linear parameters as m.linX, now, we are removing that compression
%here. 


%These are the variables which we will interact with country:
% for now, price is in here, may need to remove it...

%OPTION 1: For the multionomial logit test, control for wealth effects
%using cross country averages...generates negative price coefficients on
%average...good sign
%interX = [ m.p m.p.*bsxfun(@minus, hhincmu, mean(hhincmu))  ones(m.nObs,1) hpszwt typelargecar typesuv typetruck ];

%OPTION 2: The eventual full case, where wealth effects will be in random
%coefficients...using this within the multinomial logit generates positive
%price effects...not logical. 
%interX = [ones(m.nObs,1)  hpszwtmpg]; % no type dummies at all in this version
if (strcmp(xSet,'brandDumInter_full')) | (strcmp(xSet,'brandDum_nohomedummy')) | (strcmp(xSet,'brandDumInter_full_YrCtyDum')) | (strcmp(xSet,'brandYrCtyDum_full')) 
   %In this case we don't have a constant but instead have a full set of country brand dummies.
   %In future all runs should look like this...
   interX = [hpszwtmpg ];  
   interXcounter = [hpszwtmpgDEU ];  
else
   interX = [ones(m.nObs,1),  hpszwtmpg ]; 
   interXcounter = [ones(m.nObs,1),  hpszwtmpgDEU ]; 
end

if (m.chsq==1) %Include all square terms
    interX = [interX, hpszwtmpg.^2]; % use squares: add cross-country flexibility  
    interXcounter = [interXcounter, hpszwtmpgDEU.^2];
elseif (m.chsq==2) %Include only size squared
    interX = [interX hpszwtmpg(:,2).^2];
    interXcounter = [interXcounter hpszwtmpgDEU(:,2).^2];
end

%If price is being treated as a standard RC, we want to add it to interX:
if (m.lognormprice==0)
    interX = [-m.p interX ];
    interXcounter = [-m.p interXcounter ];
    interP = [ones(size(m.p)) zeros(size(interX))];
else
    interP = zeros(size(interX));
end


ctyInteract = dummyvar(m.ctyCode); % so we use a full set of country dummies

bigX = [];
bigXcounter = [];
bigP = [];
if m.same_rc_distr == 0 % if we want the tastes of all countries to be the same (same mean taste)
    for v = 1:size(interX,2)
           bigX = [bigX bsxfun(@times, interX(:,v),ctyInteract) ];
           bigXcounter = [bigXcounter bsxfun(@times, interXcounter(:,v),ctyInteract) ];
           bigP = [bigP bsxfun(@times, interP(:,v),ctyInteract) ];
    end
else
    bigX = interX;
    bigXcounter = interXcounter;
    bigP = interP;
end

%% Constructing year dummies
% first construct year code
year_min    = min(m.year);
m.yearCode  = m.year - year_min + 1; % yearCode starts from 1
assert( length(unique(m.yearCode)) == max(m.yearCode) ) % make sure the mkt code runs from 1 to max
yearDums = dummyvar(m.yearCode);

% drop the first year dummy: 2007, since we use full country dummies 
yearDums(:,1) = []; 

%% Constructing Model Dummies. 

%%% Shengyu added: Check to see if some modelCodes do not appear in the data
% are dropped. This will cause singularity if we use dummyvar.
% So first we need to find which modelCode are missing and then drop associated 
% vectors in modelDums.
emptModelCode = [];
everyMktModelCode = [];
for i = 1:max(m.modelCode)
    if sum(m.modelCode == i) == 0
        emptModelCode = [emptModelCode, i];
        disp(sprintf('Warning: Did not observe any observations of model %d: %s', i, c2str.Models{i}));
    end
    
    if (sum(m.modelCode == i) == m.nMkts)
        everyMktModelCode = [ everyMktModelCode i];
    end
end

baseModelCode = everyMktModelCode(1); % 12 is the modelcode for AUDI A3

if isempty(emptModelCode) == 0
    fprintf('Note: This modelCode does not exist and deleted in model dummies: %d\n',emptModelCode);
end

fprintf('Note: Base model is model %d: %s\n', baseModelCode, c2str.Models{baseModelCode});

%Dummyvar matrix, for each car, 1 in column of that car's model code, 
%zeros everywhere else. 
modelDums = dummyvar(m.modelCode);
%Eliminate base model and models that don't actually exist (shouldn't be any).
% we should drop the base model dummy since we use a full set of country dummies 
assert(isempty(baseModelCode) == 0); % make sure we do drop one model dummy
modelDums(:,[baseModelCode emptModelCode]) = []; 

%% generate firm/manufactor dummies
emptFirmCode = [];
everyMktFirmCode = [];
for i = 1:max(m.firmCode)
    if sum(m.firmCode == i) == 0
        emptFirmCode = [emptFirmCode, i];
        disp(sprintf('Warning: Did not observe any observations of firm %d: %s', i, c2str.Firms{i}));
    end
    
    if (length(unique(m.mktCode(m.firmCode == i))) == m.nMkts)
        everyMktFirmCode = [ everyMktFirmCode, i];
    end
end

%Now, find the Toyota Group code: 
for f = 1:m.nFirms
    if (strcmp(c2str.Firms{f}, 'Toyota Group'))
        ToyotaCode = f;
        break
    end
end

if ( sum(everyMktFirmCode == ToyotaCode) > 0 ) % see if Toyota is in the everymarket set
    baseFirmCode = ToyotaCode; %
else % otherwise, use the first every-market firm
    baseFirmCode = everyMktFirmCode(1); %
end

if isempty(emptFirmCode) == 0
    fprintf('Note: This firmCode does not exist and deleted in firm dummies: %d\n',emptFirmCode);
end

fprintf('Note: Base firm is firm %d: %s\n', baseFirmCode, c2str.Firms{baseFirmCode});

%Dummyvar matrix, for each car, 1 in column of that car's model code, 
%zeros everywhere else. 
firmDums = dummyvar(m.firmCode);
%Eliminate base model and models that don't actually exist (shouldn't be
%any).
% we should drop the base model dummy since we use a full set of country dummies 
assert(isempty(baseFirmCode) == 0); % make sure we do drop one model dummy
assert(isempty(emptFirmCode) == 1); % make sure firm code runs continuously
firmDums(:,[baseFirmCode emptFirmCode]) = []; 

%% generate brand dummies
emptBrandCode = [];
everyMktBrandCode = [];
for i = 1:max(m.brandCode)
    if sum(m.brandCode == i) == 0
        emptBrandCode = [emptBrandCode, i];
        disp(sprintf('Warning: Did not observe any observations of brand %d: %s', i, c2str.Brands{i}));
    end
    
    if (length(unique(m.mktCode(m.brandCode == i))) == m.nMkts)
        everyMktBrandCode = [ everyMktBrandCode, i];
    end
end

%Now, find the Toyota  code: 
for f = 1:m.nBrands
    if (strcmp(c2str.Brands{f}, 'Toyota'))
        ToyotaCode = f;
        break
    end
end

if ( sum(everyMktBrandCode == ToyotaCode) > 0 ) % see if Toyota is in the everymarket set
    baseBrandCode = ToyotaCode; %
else % otherwise, use the first every-market Brand
    baseBrandCode = everyMktBrandCode(1); %
end

if isempty(emptBrandCode) == 0
    fprintf('Note: This brandCode does not exist and deleted in brand dummies: %d\n',emptBrandCode);
end

fprintf('Note: Base brand is brand %d: %s\n', baseBrandCode, c2str.Brands{baseBrandCode});

%Dummyvar matrix, for each car, 1 in column of that car's model code, 
%zeros everywhere else. 
brandDums = dummyvar(m.brandCode);
%Eliminate base model and models that don't actually exist (shouldn't be
%any).
% we should drop the base model dummy since we use a full set of country dummies (keep France)
assert(isempty(baseBrandCode) == 0); % make sure we do drop one model dummy
assert(isempty(emptBrandCode) == 1); % make sure Brand code runs continuously
brandDums_full = brandDums;
brandDums(:,[baseBrandCode emptBrandCode]) = []; 

%% construct m.linX
%Now add the other stuff that doesn't vary by country, the home dummy and
% the model fixed effects:
switch xSet
    case 'modelDum'
          m.linX = [bigX homedummy modelDums yearDums]; %
          m.linXcounter = [bigXcounter homedummy modelDums yearDums]; %
          m.linP = [bigP zeros(size([homedummy modelDums yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'firmDum' 
          m.linX = [bigX homedummy firmDums yearDums]; % 
          m.linXcounter = [bigXcounter homedummy firmDums yearDums]; %
          m.linP = [bigP zeros(size([homedummy firmDums yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'brandDum' 
          m.linX = [bigX homedummy brandDums yearDums]; %
          m.linXcounter = [bigXcounter homedummy brandDums yearDums]; %
          m.linP = [bigP zeros(size([homedummy brandDums yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'brandDum_nohomedummy' 
          ctyDums = ctyInteract;% use a full set of country dummies
          m.linX = [bigX ctyDums brandDums yearDums]; %
          m.linXcounter = [bigXcounter ctyDums brandDums yearDums]; %
          m.linP = [bigP zeros(size([ ctyDums brandDums yearDums]))];
          m.homeDidx = size(bigX,2)+1; % we do not have home dummy in the case, but we record it like this to use it to get number of characteristics in counterfactuals     
          m.homedummy = homedummy;
    case 'brandDum_type' 
          cartype = [data.typesmallcar, data.typelargecar]; % use data.typesuv as reference, since the sum of the three is 1
          m.linX = [bigX homedummy brandDums cartype yearDums]; %
          m.linXcounter = [bigXcounter homedummy brandDums cartype yearDums]; %
          m.linP = [bigP zeros(size([homedummy brandDums cartype yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'firmDum_type' 
          cartype = [data.typesmallcar, data.typelargecar]; % use data.typesuv as reference, since the sum of the three is 1
          m.linX = [bigX homedummy firmDums cartype yearDums]; %
          m.linXcounter = [bigXcounter homedummy firmDums cartype yearDums]; %
          m.linP = [bigP zeros(size([homedummy firmDums cartype yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'brandDum_typeInter' 
          cartype = [data.typelargecar, data.typesuv];
          cartypeInter = [];
          for v = 1:size(cartype,2)
             cartypeInter = [cartypeInter bsxfun(@times, cartype(:,v), ctyInteract ) ];
          end
          m.linX = [bigX homedummy brandDums cartypeInter yearDums]; % 
          m.linXcounter = [bigXcounter homedummy brandDums cartypeInter yearDums]; % 
          m.linP = [bigP zeros(size([homedummy brandDums cartypeInter yearDums]))];
          m.homeDidx = size(bigX,2)+1; 
    case 'brandDumInter' 
          brandInter_temp = [];
          for v = 1:size(brandDums,2)
             brandInter_temp = [brandInter_temp bsxfun(@times, brandDums(:,v), ctyInteract ) ];
          end
          nonempty_brand = (sum(brandInter_temp,1) > 0);
          brandInter = brandInter_temp(:,nonempty_brand); % this is to drop the part of the dummy matrix which has no home models.
          m.linX = [bigX brandInter yearDums];  
          m.linXcounter = [bigXcounter brandInter yearDums];  
          m.linP = [bigP zeros(size([brandInter yearDums]))];
          m.startBCdums = size(bigX,2)+1;
          m.endBCdums = m.startBCdums+size(brandInter,2);
          m.homedummy = homedummy; %Kept for post-estimation projection. 
    case 'brandDumInter_full'  % this spec use a full set of brand dummies to interact with countries but do not use constant to interact with countries 
          brandInter_full_temp = [];
          for v = 1:size(brandDums_full,2)
             brandInter_full_temp = [brandInter_full_temp bsxfun(@times, brandDums_full(:,v), ctyInteract ) ];
          end
          nonempty_brand_full = (sum(brandInter_full_temp,1) > 0);
          brandInter_full = brandInter_full_temp(:,nonempty_brand_full); % this is to drop the part of the dummy matrix which has no home models.
          %ALERT: THIS IS WRONG IF WE ARE USING NORMAL? 
          m.linX = [bigX brandInter_full yearDums]; % note that in this case we need to drop the home dummy (although we still use m.homeDidx = size(bigX,2)+1 donw below -- it is useless and just to make the code consistent)           
          m.linXcounter = [bigXcounter brandInter_full yearDums];
          m.linP = [bigP zeros(size([brandInter_full yearDums]))];
          %Store indicies to find the brand-country dummies, and the home
          %dummy for post-estimation projection.
          m.startBCdums = size(bigX,2)+1;
          m.endBCdums = m.startBCdums+size(brandInter_full,2)-1;
          m.homedummy = homedummy;

    case 'brandDumInter_typeInter' 
          brandInter_temp = [];
          for v = 1:size(brandDums,2)
             brandInter_temp = [brandInter_temp bsxfun(@times, brandDums(:,v), ctyInteract ) ];
          end  
          nonempty_brand = (sum(brandInter_temp,1) > 0);
          brandInter = brandInter_temp(:,nonempty_brand); % this is to drop the part of the dummy matrix which has no home models.
          
          cartype = [data.typelargecar, data.typesuv];
          cartypeInter = [];
          for v = 1:size(cartype,2)
             cartypeInter = [cartypeInter bsxfun(@times, cartype(:,v), ctyInteract ) ];
          end       
          
          m.linX = [bigX brandInter cartypeInter yearDums]; % note that in this case we need to drop the home dummy (although we still use m.homeDidx = size(bigX,2)+1 donw below -- it is useless and just to make the code consistent)           
          m.linXcounter = [bigXcounter brandInter cartypeInter yearDums];
          m.linP = [bigP zeros(size([hpszwtmpg, hpszwtmpg.^2])) ];
          %Store indicies to find the brand-country dummies, and the home
          %dummy for post-estimation projection.
          m.startBCdums = size(bigX,2)+1;
          m.endBCdums = m.startBCdums+size(brandInter,2)-1;
          m.homedummy = homedummy;
    
    case 'modelDum_homeVar' 
          HD_temp = bsxfun(@times, homedummy,ctyInteract );
          nonempty_HD = (sum(HD_temp,1) > 0);
          homedummy_var = HD_temp(:,nonempty_HD); % this is to drop the part of the dummy matrix which has no home models.
          m.linX = [bigX homedummy_var modelDums yearDums];
          m.linXcounter = [bigXcounter homedummy_var modelDums yearDums];
          m.linP = [bigP zeros(size([homedummy_var modelDums yearDums])) ];
          %Store indicies to find the brand-country dummies, and the home
          %dummy for post-estimation projection.
          m.startHCdums = size(bigX,2)+1;
          m.endHCdums = m.startBCdums+size(homedummy_var,2)-1;  
          
    case 'brandDumInter_full_YrCtyDum'  % this spec use a full set of brand dummies to interact with countries but do not use constant to interact with countries 
          brandInter_full_temp = [];
          for v = 1:size(brandDums_full,2)
             brandInter_full_temp = [brandInter_full_temp bsxfun(@times, brandDums_full(:,v), ctyInteract ) ];
          end
          nonempty_brand_full = (sum(brandInter_full_temp,1) > 0);
          brandInter_full = brandInter_full_temp(:,nonempty_brand_full); % this is to drop the part of the dummy matrix which has no home models.
          
          % generate a set of year and country dummies -- we drop the last
          % one as the baseline.
          yearDums_full = dummyvar(m.yearCode);
          YrCtyInter_full = [];
          for v = 1:size(yearDums_full,2)
             YrCtyInter_full = [YrCtyInter_full, bsxfun(@times, yearDums_full(:,v), ctyInteract ) ];
          end
          YrCtyInter_full = YrCtyInter_full(:, (sum(YrCtyInter_full,1) > 0)); % drop the ones with empty observation
          
          % Here we need special treatment: drop ones that is already
          % captured by the brand-country dummies. This happens when a
          % brand only appear in one country for one year.
          YrCtyDum = [];
          for v = 1:size(YrCtyInter_full,2)
             if rank([brandInter_full, YrCtyDum, YrCtyInter_full(:,v)]) == size([brandInter_full, YrCtyDum, YrCtyInter_full(:,v)],2)
                 YrCtyDum = [YrCtyDum, YrCtyInter_full(:,v)];
             end
          end

          m.linX = [bigX brandInter_full YrCtyDum]; % note that in this case we need to drop the home dummy (although we still use m.homeDidx = size(bigX,2)+1 donw below -- it is useless and just to make the code consistent)           
          m.linXcounter = [bigXcounter brandInter_full YrCtyDum];
          m.linP = [bigP zeros(size([brandInter_full YrCtyDum]))];
          %Store indicies to find the brand-country dummies, and the home
          %dummy for post-estimation projection.
          m.startBCdums = size(bigX,2)+1;
          m.endBCdums = m.startBCdums+size(brandInter_full,2)-1;
          m.homedummy = homedummy;
          
    case 'brandYrCtyDum_full'  % this spec use a full set of brand dummies to interact with countries but do not use constant to interact with countries 
          brandInter_full_temp = [];
          for v = 1:size(brandDums_full,2)
             brandInter_full_temp = [brandInter_full_temp bsxfun(@times, brandDums_full(:,v), ctyInteract ) ];
          end
          nonempty_brand_full = (sum(brandInter_full_temp,1) > 0);
          brandInter_full = brandInter_full_temp(:,nonempty_brand_full); % this is to drop the part of the dummy matrix which has no home models.
          
          % Now interact the above dummes with each of the year dummy.
          yearDums_full = dummyvar(m.yearCode);
          BrandYrCtyInter_full = [];
          for v = 1:size(yearDums_full,2)
             BrandYrCtyInter_full = [BrandYrCtyInter_full, bsxfun(@times, yearDums_full(:,v), brandInter_full) ];
          end
          BrandYrCtyInter_full = BrandYrCtyInter_full(:, (sum(BrandYrCtyInter_full,1) > 0)); % drop the ones with empty observation

          m.linX = [bigX BrandYrCtyInter_full]; % note that in this case we need to drop the home dummy (although we still use m.homeDidx = size(bigX,2)+1 donw below -- it is useless and just to make the code consistent)           
          m.linXcounter = [bigXcounter BrandYrCtyInter_full];
          m.linP = [bigP zeros(size([BrandYrCtyInter_full]))];
          %Store indicies to find the brand-country dummies, and the home
          %dummy for post-estimation projection.
          m.startBCdums = size(bigX,2)+1;
          m.endBCdums = m.startBCdums+size(BrandYrCtyInter_full,2)-1;
          m.homedummy = homedummy;      
          
    otherwise
       fprintf('Error: Unrecognized X Set Requested: %s \n', xSet);
       return;
end
      
if isfield(m,'homeDidx')
    fprintf('The Home Dummy Coefficient is (from) %d\n', m.homeDidx);
end

%Safe a bit of space and clean
m.linP = logical(m.linP);
m.rcP  = logical(m.rcP);

% take in the added variables
m.homeowneddummy = homeowneddummy;
m.domestic_assembly_by_brand_dummy = cData.domesticassemblybybranddummy;
m.domestic_assembly_by_brand_num = cData.domesticassemblybybrandnum;

clear interX interXcounter bigX bigXcounter;

%% INSTRUMENTS: Same as linear coefficients, but more of them...
%
% NOTE: the instruments for horsepower, size and weight were standardized
% by stata to be mean zero variance one, sticking with that here. 

% blpIV  = [ ivowncon ivownhppwt ivownsize ivownmpgcty ...
%            ivrivcon ivrivhppwt ivrivsize ivrivmpgcty];

% note: here ctyInteract may only contain 8 counties (exclude France),
% depending on we are keeping France or normlizing France.
% But either 8 countries and 9 countries should be fine, as iv's.

%THIS IS HORRIBLE OBFUSCATION! 
%FIRST: YOU WILL CONFUSE linX and m.linX,
%SECOND: you don't really know where price is. I don't think this actually
%works when you used 
%linX_exog = m.linX;
%if (m.lognormprice==0)
%    linX_exog = linX_exog(:,m.nCtys+1:end);
%end 

%I'm replacing it with this, and replaced linx with linX_exog below
%Use Exogeounous X variables as instruments (everything but price...)
exogX = logical(1-(sum(m.linP,1)>0));
linX_exog = m.linX(:,exogX); 

% Here we need to standardize the ivs, other wise the weight matix will be
% badly scaled. 
% % This is the standardization on 4 March, 2015 (all result generated in
% % that day use this)
% ivnumprodloc = ( ivnumprodloc ) ./ repmat(std(ivnumprodloc,1), m.nObs,1); % standardized
% iv_mindist_ns = ( iv_mindist_ns ) ./ repmat(std(iv_mindist_ns,1), m.nObs,1); % standardized

% Here on 5 March 2015, we use another standardization (and for more
% variables), to make the mean as 1 and standard deviation as 1/nCtys.
% The purpose is to increase the weight of cost IVs in the estimation. In
% the previous estimation, the weight of cost IV is only 1/100 of other
% variables in m.IV.
% So on this day, we are experimenting with two changes: 
% 1) use the following new standardization;
% 2) use identity weight matrix in the 1st gmm.
% An alternative is to rise the weight of cost ivs manually (after weight
% is constructed, rise it by 100 time) -- this is implimented in this
% version (see getDesign.m)
ivnumprodloc = ( ivnumprodloc -mean( ivnumprodloc) + std(ivnumprodloc)) ./ repmat(std(ivnumprodloc), m.nObs,1); % standardized
iv_mindist_ns = ( iv_mindist_ns -mean( iv_mindist_ns) + std(iv_mindist_ns)) ./ repmat(std(iv_mindist_ns), m.nObs,1); % standardized
iv_log_mindist_wage = ( iv_log_mindist_wage -mean(  iv_log_mindist_wage) + std(iv_log_mindist_wage)) ./ repmat(std( iv_log_mindist_wage), m.nObs,1); % standardized
iv_mindist_tariff = ( iv_mindist_tariff -mean( iv_mindist_tariff) + std(iv_mindist_tariff)) ./ repmat(std(iv_mindist_tariff), m.nObs,1); % standardized

% This coefficient is for adjustment of the weight on cost IVs compared
% with other part of m.IV.
adj = 0.1;

switch ivSet
    case '2cost'
        interCostIV = [ivmindist ivnumprodloc];
        m.IV = [interCostIV linX_exog]; % since first m.nCtys columns are for prices 
    case 'all2costInter'
        interCostIV = [ivmindist ivnumprodloc];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [costIV linX_exog]; % since first m.nCtys columns are for prices  
    case '1costInter1cost'
        interCostIV = [ivmindist];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [ivnumprodloc costIV linX_exog]; % since first m.nCtys columns are for prices   
    case '1cost1costInter'
        interCostIV = [ivnumprodloc];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [ivmindist costIV linX_exog]; % since first m.nCtys columns are for prices   
    case '4blp_1costInter1cost'
        interCostIV = [ivmindist];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [blpIV(:,1:4) ivnumprodloc costIV linX_exog]; % since first m.nCtys columns are for prices   
    case '3blp_1costInter1cost'
        interCostIV = [ivmindist];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [blpIV(:,6:8) ivnumprodloc costIV linX_exog]; % since first m.nCtys columns are for prices   
    case 'other1costSquareInter'
        interCostIV = [ivnumprodloc ivnumprodloc.^2];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [costIV linX_exog]; % since first m.nCtys columns are for prices          
    case '8blp_2cost'
        costIV = [ivmindist ivnumprodloc];
        m.IV = [blpIV costIV linX_exog];
    case '8blp_2costSqure'
        costIV = [ivmindist ivnumprodloc ivmindist.^2 ivnumprodloc.^2];
        m.IV = [blpIV costIV linX_exog];
    case '8blp_1cost'
        costIV = [ivmindist];
        m.IV = [blpIV costIV linX_exog];
    case '8blp_1costSquare'
        costIV = [ivmindist  ivmindist.^2];
        m.IV = [blpIV costIV linX_exog];    
    case '4blpInter_2cost' 
        blpIV_inter = [];
        n_inter = 4; % interact first n_inter columns of blpIV
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,v),ctyInteract) ];
        end
        costIV = [ivmindist ivnumprodloc];
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,n_inter+1:end) costIV linX_exog]; 
    case '3blpInter_2cost'
        blpIV_inter = [];
        n_inter = 3; % interact last n_inter columns of blpIV: do not use ivrivcon
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,end-v),ctyInteract) ];
        end
        costIV = [ivmindist ivnumprodloc];    
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,1:n_inter) costIV linX_exog];  
    case '8blp_2costInter'  
        interCostIV = [ivmindist ivnumprodloc];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [blpIV costIV linX_exog]; % since first m.nCtys columns are for prices
    case '4blpInter_1cost' 
        blpIV_inter = [];
        n_inter = 4; % interact first n_inter columns of blpIV
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,v),ctyInteract) ];
        end
        costIV = [ivmindist];
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,n_inter+1:end) costIV linX_exog]; 
    case '3blpInter_1cost'
        blpIV_inter = [];
        n_inter = 3; % interact last n_inter columns of blpIV: do not use ivrivcon
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,end-v),ctyInteract) ];
        end
        costIV = [ivmindist];    
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,1:n_inter) costIV linX_exog];  
    case '8blp_1costInter'  
        interCostIV = [ivmindist];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [blpIV costIV linX_exog]; % since first m.nCtys columns are for prices   
    case '4blpInter_1costSquare' 
        blpIV_inter = [];
        n_inter = 4; % interact first n_inter columns of blpIV
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,v),ctyInteract) ];
        end
        costIV = [ivmindist, ivmindist.^2];
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,n_inter+1:end) costIV linX_exog]; 
    case '3blpInter_1costSquare'
        blpIV_inter = [];
        n_inter = 3; % interact last n_inter columns of blpIV: do not use ivrivcon
        for v = 1:n_inter
               blpIV_inter = [blpIV_inter bsxfun(@times, blpIV(:,end-v),ctyInteract) ];
        end
        costIV = [ivmindist, ivmindist.^2];    
        % this is just to make the iv set as the same as normal
        % distribution specification:
        m.IV = [blpIV_inter blpIV(:,1:n_inter) costIV linX_exog];  
    case '8blp_1costSquareInter'  
        interCostIV = [ivmindist, ivmindist.^2];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [blpIV costIV linX_exog]; % since first m.nCtys columns are for prices     
   case '5newCostIV'  
        %costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_log_mindist_wage, iv_mindist_tariff]; 
        %costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_mindist_wage, iv_mindist_tariff]; 
        %costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_mindist_tariff]; % first
        %costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_domestic,  iv_mindist_wage, iv_mindist_tariff]; 
        %costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_ns.^2, iv_mindist_contig, iv_mindist_domestic]; 
        costIV = [ivnumprodloc, ivnumprodloc.^2, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic]; % last
        % standardize the cost IV's
        %costIV = ( costIV - repmat(mean(costIV,1), m.nObs,1) ) ./ repmat(std(costIV,1), m.nObs,1);
 
        m.IV = [adj*costIV linX_exog]; 
    case '6newCostIV'   
        costIV = [ivnumprodloc, ivnumprodloc.^2, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_mindist_tariff]; %
        % standardize the cost IV's
        %costIV = ( costIV - repmat(mean(costIV,1), m.nObs,1) ) ./ repmat(std(costIV,1), m.nObs,1);
        m.IV = [adj*costIV linX_exog];      
    case '4blp_3newCostIV' 
        costIV = [ivnumprodloc,  iv_mindist_ns,  iv_mindist_domestic]; 
        m.IV = [blpIV(:,1:4) adj*costIV linX_exog]; 
    case '3blp_5newCostIV'
        costIV = [ivnumprodloc, ivnumprodloc.^2, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic]; 
        m.IV = [blpIV(:,end-2:end) adj*costIV linX_exog];      
    case '1blp_5newCostIV' 
        costIV = [ivnumprodloc, ivnumprodloc.^2, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic]; 
        m.IV = [blpIV(:,1) adj*costIV linX_exog];        
    case '4newCostIV' 
        costIV = [ivnumprodloc, iv_mindist_ns,  iv_mindist_domestic, iv_mindist_tariff]; 
        m.IV = [adj*costIV linX_exog];   
     case '4newCostIV_Inter'  
        interCostIV = [ivnumprodloc, iv_mindist_ns,  iv_mindist_domestic, iv_mindist_tariff];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [adj*costIV linX_exog]; % since first m.nCtys columns are for prices       
     case '4newCostIV_PartInter'  
        interCostIV = [ivnumprodloc, iv_mindist_ns];
        %interCostIV = ( interCostIV - repmat(mean(interCostIV,1), m.nObs,1) ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        %interCostIV = ( interCostIV ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        %m.IV = [costIV iv_mindist_domestic, iv_mindist_tariff  linX]; % since first m.nCtys columns are for prices   
        
        costIV  = [costIV iv_mindist_domestic, iv_mindist_tariff];
        m.IV = [adj*costIV  linX_exog]; % since first m.nCtys columns are for prices   
        
      case '3newCostIV_Inter' % this one does not use tariff, since it is highly correlated with distance 
        interCostIV = [ivnumprodloc, iv_mindist_ns,  iv_mindist_domestic];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        m.IV = [adj*costIV linX_exog]; % since first m.nCtys columns are for prices      
      case '2newCostIV_Inter' % this one does not use tariff, since it is highly correlated with distance 
        interCostIV = [ivnumprodloc, iv_mindist_ns];
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        costIV = [costIV, iv_mindist_domestic];
        m.IV = [adj*costIV linX_exog]; % since first m.nCtys columns are for prices            
    case '8blp'
        m.IV = [blpIV  linX_exog];
    case '1blp_5newCost' 
        costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_log_mindist_wage, iv_mindist_tariff]; 
        m.IV = [blpIV(:,2) adj*costIV linX_exog]; 
    case '4blp_5newCost' 
        costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_log_mindist_wage, iv_mindist_tariff]; 
        m.IV = [blpIV(:,1:4) adj*costIV linX_exog]; 
    case '3blp_5newCost' 
        costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_log_mindist_wage, iv_mindist_tariff]; 
        m.IV = [blpIV(:,6:end) adj*costIV linX_exog]; 
    case '8blp_5newCost' 
        costIV = [iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_log_mindist_wage, iv_mindist_tariff]; 
        m.IV = [blpIV adj*costIV linX_exog];        
    case '5newCostIV_v1'  
        costIV = [ivnumprodloc  iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic, iv_mindist_tariff]; 
        m.IV = [adj*costIV linX_exog];        
     case 'blp_Cost_lowcorr'  % this gives the lowest correlation between ivs, so we do not lose number of ivs
         % tariff is highly correlated with min distance, wage can be used but not sure if it is a vaild iv or not
         % num of production seems the best iv from cost side, but try not
         % to use both itself and
         % square of it since the correlation is high.
        costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic]; 
        m.IV = [blpIV(:,[3,5,8]) adj*costIV linX_exog];   
     case 'blp_Cost_lowcorr_Inter'  % this gives the lowest correlation between ivs, so we do not lose number of ivs
         % And it is interacted with countries to increase the number of
         % ivs.
         % We drop one of the dummy iv -- contig and domestic -- to aviod
         % to use too many dummies.
        costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_domestic]; 
        IV_toInter = [blpIV(:,[3,5,8]) costIV ];    
        myIV = [];
        for v = 1:size(IV_toInter,2)
               myIV = [myIV bsxfun(@times, IV_toInter(:,v),ctyInteract) ];
        end
        m.IV = [adj*myIV  linX_exog]; % since first m.nCtys columns are for prices   
      case 'Cost_lowcorr'  % this gives the lowest correlation between ivs on the cost side
        costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_contig, iv_mindist_domestic,iv_log_mindist_wage]; 
        m.IV = [adj*costIV linX_exog]; 
    case 'Cost_lowcorr_selfInter'  % this gives the lowest correlation between ivs on the cost side
        %costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_contig,ivnumprodloc.*iv_mindist_contig,iv_mindist_domestic,iv_log_mindist_wage];  % use in job880
        %costIV = [ivnumprodloc, iv_mindist_ns, iv_mindist_contig, ivnumprodloc.*iv_mindist_contig, iv_mindist_domestic, ivnumprodloc.*iv_mindist_domestic, iv_log_mindist_wage]; 
        costIV = [ivnumprodloc, iv_mindist_ns, ivnumprodloc.* iv_mindist_ns, iv_mindist_contig, ivnumprodloc.*iv_mindist_contig, iv_mindist_domestic, ivnumprodloc.*iv_mindist_domestic]; 
        m.IV = [adj*costIV linX_exog];  
    case 'DiffIV_minDist_prodLoc'   
        %Try the 
        DiffIVs = zscore( [getDiffIvs(m.rcX(:,2:end), m.rcX, m.ctyCode, 8) iv_mindist_ns ivnumprodloc iv_mindist_domestic, iv_mindist_tariff] );
        m.IV = [DiffIVs linX_exog];
    case 'DiffIV_4NCPI'
          interCostIV = [ivnumprodloc, iv_mindist_ns];
        %interCostIV = ( interCostIV - repmat(mean(interCostIV,1), m.nObs,1) ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        %interCostIV = ( interCostIV ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        %m.IV = [costIV iv_mindist_domestic, iv_mindist_tariff  linX]; % since first m.nCtys columns are for prices   
        
        costIV  = [costIV iv_mindist_domestic, iv_mindist_tariff];
        
        DiffIVs = zscore( [getDiffIvs(m.rcX(:,2:end), m.rcX, m.ctyCode, 8) ] );
        m.IV = [adj*costIV DiffIVs linX_exog];
    case '4newCostIV_PartInter_and_nearbyDiffIV'  
        %Our tradititonal instrument set plus our take on Ghandi-Houde
        %Differentiation IVs...(6 at last count for a total of 26
        %Instruments)
        
        interCostIV = [ivnumprodloc, iv_mindist_ns];
        %interCostIV = ( interCostIV - repmat(mean(interCostIV,1), m.nObs,1) ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        %interCostIV = ( interCostIV ) ./ repmat(std(interCostIV,1), m.nObs,1); % standardized
        
        costIV = [];
        for v = 1:size(interCostIV,2)
               costIV = [costIV bsxfun(@times, interCostIV(:,v),ctyInteract) ];
        end
        %m.IV = [costIV iv_mindist_domestic, iv_mindist_tariff  linX]; % since first m.nCtys columns are for prices   
        
        costIV  = [costIV iv_mindist_domestic, iv_mindist_tariff];
        
        nearbyDiffIVs = getNearbyDiffIvs(m.rcX(:,2:end), m.mktCode);
        
        m.IV = [adj*costIV nearbyDiffIVs linX_exog]; % since first m.nCtys columns are for prices   
        
    case '4costIV_and_nearbyDiffIV'  
        %Like the above case except not interacting by country- 4+6 = 10 instruments total.         
        costIV  = [ivnumprodloc, iv_mindist_ns, iv_mindist_domestic, iv_mindist_tariff];
        
        nearbyDiffIVs = getNearbyDiffIvs(m.rcX(:,2:end), m.mktCode);
        
        m.IV = [adj*costIV nearbyDiffIVs linX_exog]; % since first m.nCtys columns are for prices   
    case 'blp_4costIV_and_nearbyDiffIV'  
        %4 "BLP" instruments (but not distinguising rivals which causes multicolinearirty.
        %4 "Cost" instruments 
        %6 "Differentiation Instruments = a 1sd and .1sd ring on proximity
        %of markets by each characheristic
        %
        % Total of 14 IVs (used primarily with 11 nonlinear parameters for
        % 3 degress of freedom)
        costIV  = [ivnumprodloc, iv_mindist_ns, iv_mindist_domestic, iv_mindist_tariff];
        
        nearbyDiffIVs = getNearbyDiffIvs(m.rcX(:,2:end), m.mktCode);
        
        %We've also tried this without the hppwt and mpD instruments,
        %although we're trying including them since they don't seem to
        %cause a big multicolinearity issue.
        %m.IV = [zscore([ivowncon+ivrivcon ivownsize+ivrivsize costIV nearbyDiffIVs]) linX_exog]; % since first m.nCtys columns are for prices   
        
        m.IV = [zscore([ivowncon+ivrivcon ivownhppwt+ivrivhppwt ivownsize+ivrivsize ivownmpDcty+ivrivmpDcty costIV nearbyDiffIVs]) linX_exog];
        
    otherwise
       fprintf('Error: Unrecognized Instrument Set Requested: %s \n', ivSet);
       return;
end


% record the dimension of cost IVs, and then in getDesign, we are going to
% increase the weight on cost IVs. (It is important to keep cost IV as the first part of m.IV)
if (exist('costIV', 'var') == 1)
     m.nCostIV = size(costIV,2); % However, this is not perfect since some cost side instruments may be not included in costIV (see '4newCostIV_PartInter' for example) -- we can fix this later.
end


%Adding this to normalize instruments one last time
%m.IV = zscore(m.IV);

m.size_g = size(m.IV,2);

%For lognormal specification: All of the m.linX variables are valid instruments (price is now nonlinear)
%We need to identify the 7 nonlinear parameters...we're using 8 BLP
%instruments plus 4 cost instruments to try to do it. 

% For normal specification, price is included in m.linX, should be
% excluded.

%clear interIV bigZ
clear blpIV costIV blpIV_inter;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Now set up the Random Coefficient stuff which will be used by getMu...
%     This consists of the vector of consumer shocks for both observable
%     and unobservable charachteristics. 
% 

%We assume the dimensions of random coefficients are the width of rcX plus
%one for income and one for the price :
if (m.lognormprice==1)
   nRandomDims = size(m.rcX,2) + 2;
   % Here the +2 is for the price coefficient (log normal) and income.
else %(m.lognormprice==0)
   nRandomDims = size(m.rcX,2) + 1;
   %Here the + 1 is for income only.
end
   
% this is to decide which integration method to use
switch integrationMethod
    case 'Sparse_Grid' 
        % %Load the file, found by Felix, which contains the sparse grid for 6
        % %dimensional gaussian quadrature for sparse grids. 
        % % Alas, these aren't working so well due to NEGATIVE WEIGHTS?!?
        %NOTE: HARD-CODED to SIX DIMENSIONS!!!  
        load sparse_grid_cars
        assert(size(nodes_quad,1) == nRandomDims);
    case 'GQ'
       %full grid quadrature is maybe not great, but at least there are positive
       %weights...
       tensorPoints = 3;
       [nodes_quad, quadweight] = qnwnorm(tensorPoints*ones(1,nRandomDims),zeros(1,nRandomDims),eye(nRandomDims));
       nodes_quad = nodes_quad'; % this is to make GQ grid the same dimention as sparse grids
       quadweight = quadweight';
    case 'MC'
        % Use Monte Carlo to do integration
        % Number of points is hardcoded.
        nNodes_temp = 4e+3;
        quadweight = 1./nNodes_temp*ones(1,nNodes_temp);
        rng('default');
        rng(0);
        nodes_quad = randn(nRandomDims,nNodes_temp);% generate draws for zeta and demog (income)    
    case 'Halton_Shuffled'
        nNodes = 1000; % 4e+2;
        %Notice that haltonnormSHuffle asks for points as first argument,
        %dimensions as second.
        nodes_quad = haltonNormShuffle(nNodes, nRandomDims);
        quadweight = (1/nNodes)*ones(1,nNodes);
    otherwise
       fprintf('Error: Unrecognized Integration Method Requested: %s \n', integrationMethod);
       return;
end

m.nNodes = numel(quadweight);
m.quadweight = quadweight;
%The "-1" covers the fact that we use the final dimension of randomness to
%simulate incomes.
m.zeta_nodes = nodes_quad(1:nRandomDims-1,:);


%The demographics are a bit trickier, particularly because they vary by
%market...
%
% Our demographics will be log income and log income squared..., which 
% are distributed normally by assumption. We use the final dimension of
% quadrature nodes to integrate over income, leadint two two demog_nodes
% dimensions (one for income, one for income squared), these vary by
% markets because the mean and standard deviation of income vary by market
% (actually, the standard deviation only varies by country, but mean varies
% by market (country-year). 
%
% To be consistent with zeta_nodes, the second dimension is the number of
% consumers. 
%
% NOTE: May want to stick to just income in the initial trials...
%
m.nDemogDims = 2;
m.demog_nodes = zeros(m.nDemogDims, m.nNodes, m.nMkts);
for mk = 1:m.nMkts
    mkt_imudev = unique(hhincmu(mktCode==mk));
    mkt_isig   = unique(hhincsigma(mktCode==mk));
    if (numel(mkt_imudev)~=1) || (numel(mkt_isig)~=1)
        fprintf('Warning: The distribution of income parameters for market %d are not unique\n',mk);
    end
    
    %Here is where we use nodes_quad(end,:) as the distribution of incomes.
    m.demog_nodes(1,:,mk) = mkt_imudev + mkt_isig*nodes_quad(end,:);
    m.demog_nodes(2,:,mk) = (m.demog_nodes(1,:, mk).^2);
end

% I think that to maintain reasonable scale, we may want to demean income
% and income squared, but I'm not sure. The main benefit is that it would
% be easy to interpret the sign of the price coefficient. 
%
% May want to hold off on this in the initial trials....
% Shuts down normalization
%m.demog_norm  = [0 ; 0];
%Arbitrarily makes baselone consumer have income exp(10)...
m.demog_norm = [10 ; 100];
m.demog_nodes = bsxfun(@minus, m.demog_nodes, m.demog_norm);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Whether contraction should read from file are start new 
% We may want delta to be read from a file, but for starters will just use
% the multionomial logit delta, the estimation file will change these as 
% necessary.
m.delta_read  = 0;
m.delta_fname = '';
m.delta_start = log(m.share) - log(m.outshare);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Save the structure to be used by estimation routines
m.resFile = sprintf('..%cdemand_output%cRes_%s_%s', filesep, filesep, xSet, ivSet); % this is the name of the file for result to be saved

% %%%%%%%%%%%%%%%%%%%%%%%
% % Setting up a debug file, if we dont' want one just dont set m.debugFID
% m.debugFileName = sprintf('..%cdemand_output%cdebug.txt', filesep, filesep); %This is the name of our debug file
% m.debugFID = fopen(m.debugFileName,'w');
% assert(m.debugFID >= 0);

end
